//+-----------------------------------------------------------------------+
//|                                                !xMeter_MultiPairs.mq4 |
//|                                                                       |
//| This is an OpenSource effort collaborated by many people at TSD-Forum |
//|                                                EA Originator: FerruFx |
//|   Price Meter System GPL by Rlinac (TSD-forum) modified by jlust286 |
//|                2007/05/16 Modified by Robert Hill (IBFX mini Account) |
//|                       2008/10/20 Modified by jlust286 (Trading Rules) |
//+-----------------------------------------------------------------------+
// 
//EA SETUP:
// 1. Make sure using at least MT4 build 202.
// 2. Place the !xMeter.mq4 at experts folder.
// 3. Restart your terminal.
// 4. Show All Symbols from Market watch by pressing Ctrl+M or from menu View->Market Watch.
// 5. Right click on Market Watch dialog then select Show All.
// 6. Attach EA to ONE chart.  Works on any chart, any timeframe, the results will be the same.  
// 7. Select which currencies you want to trade using the EA properties and check "Allow live trading".
//
//HOW TO READ THE !xMETER: (This tool based on Tom Yeomans's currency meter with modifications.)
// 1. Find minimum and maximum value from each currency.
// 2. Suppose GBP has 7.4 and USD has 1.6. It tells us that GBP is stronger than USD. 
//    That means we should consider take BUY order at GBPUSD instead of SHORT order.
//
//TRADING RULES:
// 1. Check if current time is at PauseTrading otherwise trading is allowed.
// 2. Check EA properties if currency pair is allowed to be traded.
// 3. Enter a trade if the xMeter difference between the Major and Minor currencies is greater than TradeTrigger.
// 4. A trade may be closed 5 ways.  
//    - xMeter CloseTrigger (indicating reversal in short TF).
//    - Breakeven Stop Loss.
//    - Dynamic Trailing Stop.
//    - Dynamic Take Profit.
//    - Breakeven in Recovery Mode
// 5. If the currency's trades never reach Breakeven then they will be closed when the xMeter difference 
//    between currencies is less than or equal to CloseTrigger using only last 1/3 of xMeter lookback period.
// 6. If the trade become profitable then it will place a stop at Breakeven and trail with a DynamicStop.
// 7. If the difference between currencies falls to RecoverTrigger then another trade will be opened in that 
//    currency in the same direction and will open another if the currency continues to go against the trades.
//    The trades will be closed when they average breakeven.
// 8. If the trade reaches Spread+StdDev*3 it is closed by Dynamic TakeProfit.
//
//HOW TO BACKTEST:
// 1. This EA uses data from all currency pairs so you must download all data from all currency pairs first.
//    - Press F2 to open History Center.  
//    - Click on a currency pair and click download.
//    - Repeat for each currency pair until all data is downloaded for each currency pair.
//    - You can only accurately backtest as far back as you have data for all pairs.
// 2. Backtest like normal, except you can only backtest one pair at a time even though the EA can trade many pairs.
//    
//NOTES:
// 1. This tool can run on any platfrom regardless your type of account. IBFX platform also supported. 
// 2. The xMeter indicator gathers data from all availble currency pairs.  The indicator will be more reliable if 
//    you use a broker that offers all 23 currency pairs that this EA gathers data from.
// 3. Please make any changes as needed.
//
//MODIFICATIONS, ADDITIONS, CHANGES, BUG FIXES:
//    Changes by jlust286 (www.stiglu.com) on 2008/10/20:
//       - Added MoneyManagment.
//       - Added PauseTrading with default set to pause Friday and allow trades beginning Monday.
//       - Added RecoverTrigger and CloseTrigger.
//       - Added BreakEven and DynamicStop.
//       - Added DynamicTakeProfit.
//       - Added TemporarySLTP feature to prevent massive account loss incase of internet disconnection.
//       - Added grid averaging with increasing lots so additional trades can be scaled in at better prices.
//       - Changed xMeter to lookback specified bars instead of Midnight of current day.
//       - Modified xMeter so the last one third of xMeterLookBackPer is weighted 33% heavier than first two thirds. 
//       - Changed xMeter to use all available currency pairs to evaluate a currency's strength.
//       - Modified EA to be backtestable.


//================================================================================================================================
//================================================================================================================================
//---- Trade parameters
extern string    EAversion           = "!xMeter MPGA v1.6_";
extern string    label_0             = "=== Trade parameters ===";
extern double    AccountRiskPercent  =          0.1; //
extern int       xMeterLookBackPer   =           84; //How many hours should we look back to calculate High and Low for xMeter?
extern double    TradeTrigger        =          3.5; //Value must be greater than RecoverTrigger and less than 9.  Will enter trade when currencies xMeter difference is greater than TradeTrigger.
extern double    RecoverTrigger      =            0; //Will allow another trade in same currency if xMeter falls below RecoverTrigger then goes back up to TradeTrigger.
extern double    CloseTrigger        =           -5; //Closes all trades in that currency.
extern bool      AccountIsIBFXmini   =        false;
extern bool      MicroAccount        =        false;

extern string    label_1             = "=== Pairs Selection ===";
extern bool      Trade_EURUSD        =         true;
extern bool      Trade_GBPUSD        =         true;
extern bool      Trade_AUDUSD        =         true;
extern bool      Trade_USDJPY        =         true;
extern bool      Trade_USDCHF        =         true;
extern bool      Trade_USDCAD        =         true;
extern bool      Trade_EURJPY        =         true;
extern bool      Trade_EURGBP        =         true;
extern bool      Trade_EURCHF        =         true;
extern bool      Trade_EURAUD        =         true;
extern bool      Trade_GBPJPY        =         true;
extern bool      Trade_GBPCHF        =         true;

//Internal Variables
int       MagicNumber         =       821962;
int       Slippage            =            2; //Slippage 
int       PauseStartTime      =        42300; //PauseTrading DayofWeek,Hour,Minute. example input: 50700 will pause trading on friday at 7:00 am.  Current trades will continue until CloseTrigger reached or stopped out. Set to 0 to disable.
int       PauseEndTime        =        10500; //End pause trading.  example input: 10800 will allow trading after 8:00am on monday.

//================================================================================================================================
//================================================================================================================================
//---- !xMeter indicator settings
#define ARRSIZE  23                     // number of pairs !!!DON'T CHANGE THIS NUMBER!!!
#define PAIRSIZE 7                      // number of currencies !!!DON'T CHANGE THIS NUMBER!!!
#define TABSIZE  10                     // scale of currency's power !!!DON'T CHANGE THIS NUMBER!!!
#define ORDER    2                      // available type of order !!!DON'T CHANGE THIS NUMBER!!!
// Currency pair
#define EURUSD 0
#define GBPUSD 1
#define AUDUSD 2
#define USDJPY 3
#define USDCHF 4
#define USDCAD 5
#define EURJPY 6
#define EURGBP 7
#define EURCHF 8
#define EURAUD 9
#define GBPJPY 10
#define GBPCHF 11

#define NZDUSD 12
#define EURCAD 13
#define EURNZD 14
#define CHFJPY 15
#define AUDCHF 16
#define AUDCAD 17
#define CADCHF 18
#define CADJPY 19
#define AUDJPY 20
#define AUDNZD 21
#define NZDJPY 22

// Currency
#define USD 0
#define EUR 1
#define GBP 2
#define CHF 3
#define CAD 4
#define AUD 5
#define JPY 6

string aPair[ARRSIZE]   = {"EURUSDm","GBPUSDm","AUDUSDm","USDJPYm","USDCHFm","USDCADm","EURJPYm","EURGBPm","EURCHFm","EURAUDm","GBPJPYm","GBPCHFm"
                           ,"NZDUSDm","EURCADm","EURNZDm","CHFJPYm","AUDCHFm","AUDCADm","CADCHFm","CADJPYm","AUDJPYm","AUDNZDm","NZDJPYm"};
string aMajor[PAIRSIZE] = {"USD","EUR","GBP","CHF","CAD","AUD","JPY"};
string aOrder[ORDER]    = {"BUY ","SELL "};

double aMeter[PAIRSIZE];
double aHigh[ARRSIZE];
double aLow[ARRSIZE];
double aBid[ARRSIZE];
double aAsk[ARRSIZE];
double aRatio[ARRSIZE];
double aRange[ARRSIZE];
double aMajorSymb[ARRSIZE];
double aMinorSymb[ARRSIZE];

double bMeter[PAIRSIZE];
double bHigh[ARRSIZE];
double bLow[ARRSIZE];
double bBid[ARRSIZE];
double bAsk[ARRSIZE];
double bRatio[ARRSIZE];
double bRange[ARRSIZE];
double bMajorSymb[ARRSIZE];
double bMinorSymb[ARRSIZE];
double bRatioWeighted[ARRSIZE];


int    index, MaxSpread_EURUSD, MaxSpread_GBPUSD, MaxSpread_AUDUSD, MaxSpread_USDJPY, MaxSpread_USDCHF, MaxSpread_USDCAD, MaxSpread_EURJPY, MaxSpread_EURGBP, MaxSpread_EURCHF, MaxSpread_EURAUD, MaxSpread_GBPJPY, MaxSpread_GBPCHF;
double tempTP, tempSL, sl, tp, lot, InitialLotSize, lotPrecision, WeightedAverage, LastErrorTime;
int    RecoveryAmount, direction, cycle, tick, dot, cnt, USDdivisor, EURdivisor, AUDdivisor, GBPdivisor, CADdivisor, CHFdivisor, JPYdivisor;
bool   PauseTrading, Initialized=false, Alert1=false, AlertNOW;
string message, dots, symb, Alert2, Time1, a="!";
#define  NL "\n"


//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
{
     
   for(int cnt=0;cnt<OrdersTotal();cnt++)
   {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if(OrderMagicNumber()==MagicNumber){AlertNOW=True; LastErrorTime=TimeCurrent();}
   }      

   //Filter News Announcemnets
   MaxSpread_EURUSD=MarketInfo("EURUSD",MODE_SPREAD);
   MaxSpread_GBPUSD=MarketInfo("GBPUSD",MODE_SPREAD);
   MaxSpread_AUDUSD=MarketInfo("AUDUSD",MODE_SPREAD);
   MaxSpread_USDJPY=MarketInfo("USDJPY",MODE_SPREAD);
   MaxSpread_USDCHF=MarketInfo("USDCHF",MODE_SPREAD);
   MaxSpread_USDCAD=MarketInfo("USDCAD",MODE_SPREAD);
   MaxSpread_EURJPY=MarketInfo("EURJPY",MODE_SPREAD);
   MaxSpread_EURGBP=MarketInfo("EURGBP",MODE_SPREAD);
   MaxSpread_EURCHF=MarketInfo("EURCHF",MODE_SPREAD);
   MaxSpread_EURAUD=MarketInfo("EURAUD",MODE_SPREAD);
   MaxSpread_GBPJPY=MarketInfo("GBPJPY",MODE_SPREAD);
   MaxSpread_GBPCHF=MarketInfo("GBPCHF",MODE_SPREAD);
   
   Alert2="EA will initialize with new tick";   
   initGraph();
   CalculateLotPrecision();
   tick=Volume[0];
   if(IsTesting())
   {
      string symb=StringSubstr(Symbol(),0,6);
      if(symb=="EURUSD")Trade_EURUSD=true;else Trade_EURUSD=false;
      if(symb=="GBPUSD")Trade_GBPUSD=true;else Trade_GBPUSD=false;
      if(symb=="AUDUSD")Trade_AUDUSD=true;else Trade_AUDUSD=false;
      if(symb=="USDJPY")Trade_USDJPY=true;else Trade_USDJPY=false;
      if(symb=="USDCHF")Trade_USDCHF=true;else Trade_USDCHF=false;
      if(symb=="USDCAD")Trade_USDCAD=true;else Trade_USDCAD=false;
      if(symb=="EURJPY")Trade_EURJPY=true;else Trade_EURJPY=false;
      if(symb=="EURGBP")Trade_EURGBP=true;else Trade_EURGBP=false;
      if(symb=="EURCHF")Trade_EURCHF=true;else Trade_EURCHF=false;
      if(symb=="EURAUD")Trade_EURAUD=true;else Trade_EURAUD=false;
      if(symb=="GBPJPY")Trade_GBPJPY=true;else Trade_GBPJPY=false;
      if(symb=="GBPCHF")Trade_GBPCHF=true;else Trade_GBPCHF=false;
      a="";
   }
   GlobalVariableSet(a+"xMeter_Initialized",1); cycle=1;
}

//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
  {
   GlobalVariableDel(a+"xMeter_Initialized");
   objectDeleteAll();      
   return(0);
  }
  
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
{  
   if(!IsTesting())//alert if EA running on another chart
   { 
      if(GlobalVariableGet(a+"xMeter_Initialized")==1 && cycle==1){GlobalVariableSet(a+"xMeter_Initialized",2);cycle=2;}
      else if(GlobalVariableGet(a+"xMeter_Initialized")==2 && cycle==2){GlobalVariableSet(a+"xMeter_Initialized",1);cycle=1;}else Alert1=true;
      if(Alert1){Alert("!xMeter EA is already trading on a different chart."); GlobalVariableSet(a+"xMeter_Initialized",1); cycle=1; Alert1=false;}
   }   
   if (tick+3<Volume[0] || IsTesting())Initialized=true;//give xMeter 3 ticks from start to gather all currency data before allowing trade.
   InitialLotSize=AutoLot();
   PauseAtTime(PauseStartTime,PauseEndTime); 
   
   //---- !xMeter calculation 
   for (index = 0; index < ARRSIZE; index++)// initialize all pairs required value 
   {
      RefreshRates();// refresh all currencies
      if (AccountIsIBFXmini) symb = aPair[index];
      else   symb  = StringSubstr(aPair[index],0,6);
      double ask   = MarketInfo(symb,MODE_ASK);
      double bid   = MarketInfo(symb,MODE_BID);
      double point = MarketInfo(symb,MODE_POINT);
      int    sprd  = MarketInfo(symb,MODE_SPREAD);

      //---- Get point info for symbol for shorter lookback period for entry confirmation and exit trigger.
      point = MarketInfo(symb,MODE_POINT); 
      int ahighBar = iHighest(symb,60,MODE_HIGH,xMeterLookBackPer/3,0);//---- Find highest bar during the last one third of lookback period
      int alowBar  = iLowest(symb,60,MODE_LOW,xMeterLookBackPer/3,0);//---- Find lowest bar during the last one third of lookback period
      aHigh[index] = iHigh(symb,60,ahighBar);//---- Set high
      aLow[index]  = iLow(symb,60,alowBar);//---- Set low 
      aAsk[index]  = iClose(symb,60,0);//---- Set close
      aRange[index]= MathMax((aHigh[index]-aLow[index])/point,1);//---- Calculate range
      aRatio[index]= (aAsk[index]-aLow[index])/aRange[index]/point;//---- Calculate pair's ratio and inverse ratio

      //---- Get point info for symbol
      int bhighBar = iHighest(symb,60,MODE_HIGH,xMeterLookBackPer,0);//---- Find highest bar during the lookback period
      int blowBar  = iLowest(symb,60,MODE_LOW,xMeterLookBackPer,0);//---- Find lowest bar during the lookback period
      bHigh[index] = iHigh(symb,60,bhighBar);//---- Set high
      bLow[index]  = iLow(symb,60,blowBar);//---- Set low 
      bAsk[index]  = iClose(symb,60,0);//---- Set close
      bRange[index]= MathMax((bHigh[index]-bLow[index])/point,1);//---- Calculate range
      bRatio[index]= (bAsk[index]-bLow[index])/bRange[index]/point;//---- Calculate pair's ratio and inverse ratio
      bRatioWeighted[index]=(bRatio[index]*2+aRatio[index])/3;//---- Weight xMeter so the last one third of LookBackPeriod is weighted 33% heavier than first two thirds. 
      
      //--- grade table for entry confirmationa and exit trigger
      int aTable[TABSIZE]  = {0,3,10,25,40,50,60,75,90,97};
      if      (aRatio[index]*100 <=aTable[0])  aMajorSymb[index]   = 0.001;
      else if (aRatio[index]*100 < aTable[1])  aMajorSymb[index]   = 0.001;
      else if (aRatio[index]*100 < aTable[2])  aMajorSymb[index]   = 1;
      else if (aRatio[index]*100 < aTable[3])  aMajorSymb[index]   = 2;
      else if (aRatio[index]*100 < aTable[4])  aMajorSymb[index]   = 3;
      else if (aRatio[index]*100 < aTable[5])  aMajorSymb[index]   = 4;
      else if (aRatio[index]*100 < aTable[6])  aMajorSymb[index]   = 5;
      else if (aRatio[index]*100 < aTable[7])  aMajorSymb[index]   = 6;
      else if (aRatio[index]*100 < aTable[8])  aMajorSymb[index]   = 7;
      else if (aRatio[index]*100 < aTable[9])  aMajorSymb[index]   = 8;
      else                                     aMajorSymb[index]   = 9;
      aMinorSymb[index] = 9-aMajorSymb[index];//---- Set a pair's grade and inverse grade

      //--- grade table for currency's power
      int bTable[TABSIZE]  = {0,3,10,25,40,50,60,75,90,97};
      if      (bRatioWeighted[index]*100 <=bTable[0])  bMajorSymb[index]   = 0.001;
      else if (bRatioWeighted[index]*100 < bTable[1])  bMajorSymb[index]   = 0.001;
      else if (bRatioWeighted[index]*100 < bTable[2])  bMajorSymb[index]   = 1;
      else if (bRatioWeighted[index]*100 < bTable[3])  bMajorSymb[index]   = 2;
      else if (bRatioWeighted[index]*100 < bTable[4])  bMajorSymb[index]   = 3;
      else if (bRatioWeighted[index]*100 < bTable[5])  bMajorSymb[index]   = 4;
      else if (bRatioWeighted[index]*100 < bTable[6])  bMajorSymb[index]   = 5;
      else if (bRatioWeighted[index]*100 < bTable[7])  bMajorSymb[index]   = 6;
      else if (bRatioWeighted[index]*100 < bTable[8])  bMajorSymb[index]   = 7;
      else if (bRatioWeighted[index]*100 < bTable[9])  bMajorSymb[index]   = 8;
      else                                     bMajorSymb[index]   = 9;
      bMinorSymb[index] = 9-bMajorSymb[index];//---- Set a pair's grade and inverse grade
      if (point>0.1){aMajorSymb[index]=0; aMinorSymb[index]=0; bMajorSymb[index]=0; bMinorSymb[index]=0;}//if broker does not offer currency pair then exclude from xMeter averaging

      double LowestBuy=1000, LowestSell=1000, HighestBuy=0, HighestSell=0, BuyProfits=0, SellProfits=0;
      int Orders=0, sellorders=0, buyorders=0;
      for(int j=0;j<OrdersTotal();j++)
      {
         OrderSelect(j, SELECT_BY_POS, MODE_TRADES);
         if(OrderMagicNumber()==MagicNumber && OrderSymbol()==symb)//---- an order is opened with same symbol, same magic number 
         {
            Orders++; 
         } 
         if(OrderMagicNumber()==MagicNumber && OrderSymbol()==symb && OrderType()==OP_BUY)//---- an order is opened with same symbol, same magic number, same direction
         {
            buyorders++;  
            if (OrderOpenPrice() < LowestBuy ) LowestBuy  = OrderOpenPrice(); 
            if (OrderOpenPrice() > HighestBuy) HighestBuy = OrderOpenPrice();
            BuyProfits=BuyProfits+(OrderProfit()+OrderSwap()+OrderCommission());
         }
         if(OrderMagicNumber()==MagicNumber && OrderSymbol()==symb && OrderType()==OP_SELL)//---- an order is opened with same symbol, same magic number, same direction
         {
            sellorders++; 
            if (OrderOpenPrice() > HighestSell) HighestSell = OrderOpenPrice(); 
            if (OrderOpenPrice() < LowestSell ) LowestSell  = OrderOpenPrice();
            SellProfits=SellProfits+(OrderProfit()+OrderSwap()+OrderCommission());
         }
      }   
      double sdv=iStdDev(symb,15,6,0,0,0,0);
      if(buyorders ==0)GlobalVariableSet(a+"xMeter_"+symb+"buy",1);                                   
      if(sellorders==0)GlobalVariableSet(a+"xMeter_"+symb+"sell",1);  
      if(buyorders>1)
      {
         if(AccountProfit()>0)ExitAllTradesNOW();
         if(BuyProfits>0)CloseBuysNOW(symb);
      }         
      if(sellorders>1)
      {
         if(AccountProfit()>0)ExitAllTradesNOW();
         if(SellProfits>0)CloseSellsNOW(symb);  
      }    
      DynamicBuyTP(symb,buyorders,ask,bid,point,sprd);
      DynamicSellTP(symb,sellorders,ask,bid,point,sprd);
      DynamicBuySL(symb,buyorders,ask,bid,point,sprd);
      DynamicSellSL(symb,sellorders,ask,bid,point,sprd);
      AdjustTemporarySLTP();//Set initial SL & TP incase internet disconnect only!
  }//---- for
 
  //---- calculate divisor for averaging
   USDdivisor=0; if(aMajorSymb[USDJPY]>0)USDdivisor++; if(aMajorSymb[USDCHF]>0)USDdivisor++; if(aMajorSymb[USDCAD]>0)USDdivisor++; if(aMinorSymb[EURUSD]>0)USDdivisor++; if(aMinorSymb[GBPUSD]>0)USDdivisor++; if(aMinorSymb[AUDUSD]>0)USDdivisor++; if(aMinorSymb[NZDUSD]>0)USDdivisor++; 
   EURdivisor=0; if(aMajorSymb[EURUSD]>0)EURdivisor++; if(aMajorSymb[EURJPY]>0)EURdivisor++; if(aMajorSymb[EURGBP]>0)EURdivisor++; if(aMajorSymb[EURCHF]>0)EURdivisor++; if(aMajorSymb[EURAUD]>0)EURdivisor++; if(aMajorSymb[EURCAD]>0)EURdivisor++; if(aMajorSymb[EURNZD]>0)EURdivisor++;
   GBPdivisor=0; if(aMajorSymb[GBPUSD]>0)GBPdivisor++; if(aMajorSymb[GBPJPY]>0)GBPdivisor++; if(aMajorSymb[GBPCHF]>0)GBPdivisor++; if(aMinorSymb[EURGBP]>0)GBPdivisor++;
   CHFdivisor=0; if(aMinorSymb[USDCHF]>0)CHFdivisor++; if(aMinorSymb[EURCHF]>0)CHFdivisor++; if(aMinorSymb[GBPCHF]>0)CHFdivisor++; if(aMajorSymb[CHFJPY]>0)CHFdivisor++; if(aMinorSymb[AUDCHF]>0)CHFdivisor++; if(aMinorSymb[CADCHF]>0)CHFdivisor++;
   CADdivisor=0; if(aMinorSymb[USDCAD]>0)CADdivisor++; if(aMinorSymb[EURCAD]>0)CADdivisor++; if(aMinorSymb[AUDCAD]>0)CADdivisor++; if(aMajorSymb[CADCHF]>0)CADdivisor++; if(aMajorSymb[CADJPY]>0)CADdivisor++;
   AUDdivisor=0; if(aMajorSymb[AUDUSD]>0)AUDdivisor++; if(aMinorSymb[EURAUD]>0)AUDdivisor++; if(aMajorSymb[AUDJPY]>0)AUDdivisor++; if(aMajorSymb[AUDNZD]>0)AUDdivisor++; if(aMajorSymb[AUDCAD]>0)AUDdivisor++; if(aMajorSymb[AUDCHF]>0)AUDdivisor++;
   JPYdivisor=0; if(aMinorSymb[USDJPY]>0)JPYdivisor++; if(aMinorSymb[EURJPY]>0)JPYdivisor++; if(aMinorSymb[GBPJPY]>0)JPYdivisor++; if(aMinorSymb[CHFJPY]>0)JPYdivisor++; if(aMinorSymb[AUDJPY]>0)JPYdivisor++; if(aMinorSymb[CADJPY]>0)JPYdivisor++; if(aMinorSymb[NZDJPY]>0)JPYdivisor++; 
   
   //---- calculate all currencies meter         
   aMeter[USD] = NormalizeDouble((aMajorSymb[USDJPY]+aMajorSymb[USDCHF]+aMajorSymb[USDCAD]+aMinorSymb[EURUSD]+aMinorSymb[GBPUSD]+aMinorSymb[AUDUSD]+aMinorSymb[NZDUSD])/USDdivisor,1);
   aMeter[EUR] = NormalizeDouble((aMajorSymb[EURUSD]+aMajorSymb[EURJPY]+aMajorSymb[EURGBP]+aMajorSymb[EURCHF]+aMajorSymb[EURAUD]+aMajorSymb[EURCAD]+aMajorSymb[EURNZD])/EURdivisor,1);
   aMeter[GBP] = NormalizeDouble((aMajorSymb[GBPUSD]+aMajorSymb[GBPJPY]+aMajorSymb[GBPCHF]+aMinorSymb[EURGBP])/GBPdivisor,1);
   aMeter[CHF] = NormalizeDouble((aMinorSymb[USDCHF]+aMinorSymb[EURCHF]+aMinorSymb[GBPCHF]+aMajorSymb[CHFJPY]+aMinorSymb[AUDCHF]+aMinorSymb[CADCHF])/CHFdivisor,1);
   aMeter[CAD] = NormalizeDouble((aMinorSymb[USDCAD]+aMinorSymb[EURCAD]+aMinorSymb[AUDCAD]+aMajorSymb[CADCHF]+aMajorSymb[CADJPY])/CADdivisor,1);
   aMeter[AUD] = NormalizeDouble((aMajorSymb[AUDUSD]+aMinorSymb[EURAUD]+aMajorSymb[AUDJPY]+aMajorSymb[AUDNZD]+aMajorSymb[AUDCAD]+aMajorSymb[AUDCHF])/AUDdivisor,1);
   aMeter[JPY] = NormalizeDouble((aMinorSymb[USDJPY]+aMinorSymb[EURJPY]+aMinorSymb[GBPJPY]+aMinorSymb[CHFJPY]+aMinorSymb[AUDJPY]+aMinorSymb[CADJPY]+aMinorSymb[NZDJPY])/JPYdivisor,1);
  
   bMeter[USD] = NormalizeDouble((bMajorSymb[USDJPY]+bMajorSymb[USDCHF]+bMajorSymb[USDCAD]+bMinorSymb[EURUSD]+bMinorSymb[GBPUSD]+bMinorSymb[AUDUSD]+bMinorSymb[NZDUSD])/USDdivisor,1);
   bMeter[EUR] = NormalizeDouble((bMajorSymb[EURUSD]+bMajorSymb[EURJPY]+bMajorSymb[EURGBP]+bMajorSymb[EURCHF]+bMajorSymb[EURAUD]+bMajorSymb[EURCAD]+bMajorSymb[EURNZD])/EURdivisor,1);
   bMeter[GBP] = NormalizeDouble((bMajorSymb[GBPUSD]+bMajorSymb[GBPJPY]+bMajorSymb[GBPCHF]+bMinorSymb[EURGBP])/GBPdivisor,1);
   bMeter[CHF] = NormalizeDouble((bMinorSymb[USDCHF]+bMinorSymb[EURCHF]+bMinorSymb[GBPCHF]+bMajorSymb[CHFJPY]+bMinorSymb[AUDCHF]+bMinorSymb[CADCHF])/CHFdivisor,1);
   bMeter[CAD] = NormalizeDouble((bMinorSymb[USDCAD]+bMinorSymb[EURCAD]+bMinorSymb[AUDCAD]+bMajorSymb[CADCHF]+bMajorSymb[CADJPY])/CADdivisor,1);
   bMeter[AUD] = NormalizeDouble((bMajorSymb[AUDUSD]+bMinorSymb[EURAUD]+bMajorSymb[AUDJPY]+bMajorSymb[AUDNZD]+bMajorSymb[AUDCAD]+bMajorSymb[AUDCHF])/AUDdivisor,1);
   bMeter[JPY] = NormalizeDouble((bMinorSymb[USDJPY]+bMinorSymb[EURJPY]+bMinorSymb[GBPJPY]+bMinorSymb[CHFJPY]+bMinorSymb[AUDJPY]+bMinorSymb[CADJPY]+bMinorSymb[NZDJPY])/JPYdivisor,1);

   //---- display the results     
   if(Initialized)
   {
      objectBlank();   
      paintUSD(bMeter[USD]);
      paintEUR(bMeter[EUR]);
      paintGBP(bMeter[GBP]);
      paintCHF(bMeter[CHF]);
      paintCAD(bMeter[CAD]);
      paintAUD(bMeter[AUD]);
      paintJPY(bMeter[JPY]);
      paintLine();
   }   
   
   //---- GO TRADING
   double aEUR = aMeter[EUR];
   double aUSD = aMeter[USD];
   double aGBP = aMeter[GBP];
   double aCHF = aMeter[CHF];
   double aCAD = aMeter[CAD];
   double aAUD = aMeter[AUD];
   double aJPY = aMeter[JPY];

   double bEUR = bMeter[EUR];
   double bUSD = bMeter[USD];
   double bGBP = bMeter[GBP];
   double bCHF = bMeter[CHF];
   double bCAD = bMeter[CAD];
   double bAUD = bMeter[AUD];
   double bJPY = bMeter[JPY];
      
   if ( AccountIsIBFXmini == true )
   {
      //If xMeter reaches TradeTrigger then enter the trade
      //If xMeter falls to RecoverTrigger then allow another trade in same currency pair.
      if ( Trade_EURUSD && bEUR>0 && bUSD>0 && (bEUR-bUSD>=TradeTrigger || (aEUR-aUSD<=RecoverTrigger && GlobalVariableGet(a+"xMeter_EURUSDbuy" )==2))) {ManageBuy ("EURUSDm",MaxSpread_EURUSD);}
      if ( Trade_EURUSD && bEUR>0 && bUSD>0 && (bUSD-bEUR>=TradeTrigger || (aUSD-aEUR<=RecoverTrigger && GlobalVariableGet(a+"xMeter_EURUSDsell")==2))) {ManageSell("EURUSDm",MaxSpread_EURUSD);}
      if ( Trade_GBPUSD && bGBP>0 && bUSD>0 && (bGBP-bUSD>=TradeTrigger || (aGBP-aUSD<=RecoverTrigger && GlobalVariableGet(a+"xMeter_GBPUSDbuy" )==2))) {ManageBuy ("GBPUSDm",MaxSpread_GBPUSD);}
      if ( Trade_GBPUSD && bGBP>0 && bUSD>0 && (bUSD-bGBP>=TradeTrigger || (aUSD-aGBP<=RecoverTrigger && GlobalVariableGet(a+"xMeter_GBPUSDsell")==2))) {ManageSell("GBPUSDm",MaxSpread_GBPUSD);}
      if ( Trade_AUDUSD && bAUD>0 && bUSD>0 && (bAUD-bUSD>=TradeTrigger || (aAUD-aUSD<=RecoverTrigger && GlobalVariableGet(a+"xMeter_AUDUSDbuy" )==2))) {ManageBuy ("AUDUSDm",MaxSpread_AUDUSD);}
      if ( Trade_AUDUSD && bAUD>0 && bUSD>0 && (bUSD-bAUD>=TradeTrigger || (aUSD-aAUD<=RecoverTrigger && GlobalVariableGet(a+"xMeter_AUDUSDsell")==2))) {ManageSell("AUDUSDm",MaxSpread_AUDUSD);}
      if ( Trade_USDJPY && bUSD>0 && bJPY>0 && (bUSD-bJPY>=TradeTrigger || (aUSD-aJPY<=RecoverTrigger && GlobalVariableGet(a+"xMeter_USDJPYbuy" )==2))) {ManageBuy ("USDJPYm",MaxSpread_USDJPY);}
      if ( Trade_USDJPY && bUSD>0 && bJPY>0 && (bJPY-bUSD>=TradeTrigger || (aJPY-aUSD<=RecoverTrigger && GlobalVariableGet(a+"xMeter_USDJPYsell")==2))) {ManageSell("USDJPYm",MaxSpread_USDJPY);}
      if ( Trade_USDCHF && bUSD>0 && bCHF>0 && (bUSD-bCHF>=TradeTrigger || (aUSD-aCHF<=RecoverTrigger && GlobalVariableGet(a+"xMeter_USDCHFbuy" )==2))) {ManageBuy ("USDCHFm",MaxSpread_USDCHF);}
      if ( Trade_USDCHF && bUSD>0 && bCHF>0 && (bCHF-bUSD>=TradeTrigger || (aCHF-aUSD<=RecoverTrigger && GlobalVariableGet(a+"xMeter_USDCHFsell")==2))) {ManageSell("USDCHFm",MaxSpread_USDCHF);}
      if ( Trade_USDCAD && bUSD>0 && bCAD>0 && (bUSD-bCAD>=TradeTrigger || (aUSD-aCAD<=RecoverTrigger && GlobalVariableGet(a+"xMeter_USDCADbuy" )==2))) {ManageBuy ("USDCADm",MaxSpread_USDCAD);}
      if ( Trade_USDCAD && bUSD>0 && bCAD>0 && (bCAD-bUSD>=TradeTrigger || (aCAD-aUSD<=RecoverTrigger && GlobalVariableGet(a+"xMeter_USDCADsell")==2))) {ManageSell("USDCADm",MaxSpread_USDCAD);}
      if ( Trade_EURJPY && bEUR>0 && bJPY>0 && (bEUR-bJPY>=TradeTrigger || (aEUR-aJPY<=RecoverTrigger && GlobalVariableGet(a+"xMeter_EURJPYbuy" )==2))) {ManageBuy ("EURJPYm",MaxSpread_EURJPY);}
      if ( Trade_EURJPY && bEUR>0 && bJPY>0 && (bJPY-bEUR>=TradeTrigger || (aJPY-aEUR<=RecoverTrigger && GlobalVariableGet(a+"xMeter_EURJPYsell")==2))) {ManageSell("EURJPYm",MaxSpread_EURJPY);}
      if ( Trade_EURGBP && bEUR>0 && bGBP>0 && (bEUR-bGBP>=TradeTrigger || (aEUR-aGBP<=RecoverTrigger && GlobalVariableGet(a+"xMeter_EURGBPbuy" )==2))) {ManageBuy ("EURGBPm",MaxSpread_EURGBP);}
      if ( Trade_EURGBP && bEUR>0 && bGBP>0 && (bGBP-bEUR>=TradeTrigger || (aGBP-aEUR<=RecoverTrigger && GlobalVariableGet(a+"xMeter_EURGBPsell")==2))) {ManageSell("EURGBPm",MaxSpread_EURGBP);}
      if ( Trade_EURCHF && bEUR>0 && bCHF>0 && (bEUR-bCHF>=TradeTrigger || (aEUR-aCHF<=RecoverTrigger && GlobalVariableGet(a+"xMeter_EURCHFbuy" )==2))) {ManageBuy ("EURCHFm",MaxSpread_EURCHF);}
      if ( Trade_EURCHF && bEUR>0 && bCHF>0 && (bCHF-bEUR>=TradeTrigger || (aCHF-aEUR<=RecoverTrigger && GlobalVariableGet(a+"xMeter_EURCHFsell")==2))) {ManageSell("EURCHFm",MaxSpread_EURCHF);}
      if ( Trade_EURAUD && bEUR>0 && bAUD>0 && (bEUR-bAUD>=TradeTrigger || (aEUR-aAUD<=RecoverTrigger && GlobalVariableGet(a+"xMeter_EURAUDbuy" )==2))) {ManageBuy ("EURAUDm",MaxSpread_EURAUD);}
      if ( Trade_EURAUD && bEUR>0 && bAUD>0 && (bAUD-bEUR>=TradeTrigger || (aAUD-aEUR<=RecoverTrigger && GlobalVariableGet(a+"xMeter_EURAUDsell")==2))) {ManageSell("EURAUDm",MaxSpread_EURAUD);}
      if ( Trade_GBPJPY && bGBP>0 && bJPY>0 && (bGBP-bJPY>=TradeTrigger || (aGBP-aJPY<=RecoverTrigger && GlobalVariableGet(a+"xMeter_GBPJPYbuy" )==2))) {ManageBuy ("GBPJPYm",MaxSpread_GBPJPY);}
      if ( Trade_GBPJPY && bGBP>0 && bJPY>0 && (bJPY-bGBP>=TradeTrigger || (aJPY-aGBP<=RecoverTrigger && GlobalVariableGet(a+"xMeter_GBPJPYsell")==2))) {ManageSell("GBPJPYm",MaxSpread_GBPJPY);}
      if ( Trade_GBPCHF && bGBP>0 && bCHF>0 && (bGBP-bCHF>=TradeTrigger || (aGBP-aCHF<=RecoverTrigger && GlobalVariableGet(a+"xMeter_GBPCHFbuy" )==2))) {ManageBuy ("GBPCHFm",MaxSpread_GBPCHF);}
      if ( Trade_GBPCHF && bGBP>0 && bCHF>0 && (bCHF-bGBP>=TradeTrigger || (aCHF-aGBP<=RecoverTrigger && GlobalVariableGet(a+"xMeter_GBPCHFsell")==2))) {ManageSell("GBPCHFm",MaxSpread_GBPCHF);}

      //If xMeter falls to CloseTrigger then close all trades in that currency pair.
      if (Trade_EURUSD && aEUR-aUSD<=CloseTrigger && aEUR>0 && aUSD>0) {CloseBuysNOW ("EURUSDm");}
      if (Trade_EURUSD && aUSD-aEUR<=CloseTrigger && aEUR>0 && aUSD>0) {CloseSellsNOW("EURUSDm");}
      if (Trade_GBPUSD && aGBP-aUSD<=CloseTrigger && aGBP>0 && aUSD>0) {CloseBuysNOW ("GBPUSDm");}
      if (Trade_GBPUSD && aUSD-aGBP<=CloseTrigger && aGBP>0 && aUSD>0) {CloseSellsNOW("GBPUSDm");}
      if (Trade_AUDUSD && aAUD-aUSD<=CloseTrigger && aAUD>0 && aUSD>0) {CloseBuysNOW ("AUDUSDm");}
      if (Trade_AUDUSD && aUSD-aAUD<=CloseTrigger && aAUD>0 && aUSD>0) {CloseSellsNOW("AUDUSDm");}
      if (Trade_USDJPY && aUSD-aJPY<=CloseTrigger && aUSD>0 && aJPY>0) {CloseBuysNOW ("USDJPYm");}
      if (Trade_USDJPY && aJPY-aUSD<=CloseTrigger && aUSD>0 && aJPY>0) {CloseSellsNOW("USDJPYm");}
      if (Trade_USDCHF && aUSD-aCHF<=CloseTrigger && aUSD>0 && aCHF>0) {CloseBuysNOW ("USDCHFm");}
      if (Trade_USDCHF && aCHF-aUSD<=CloseTrigger && aUSD>0 && aCHF>0) {CloseSellsNOW("USDCHFm");}
      if (Trade_USDCAD && aUSD-aCAD<=CloseTrigger && aUSD>0 && aCAD>0) {CloseBuysNOW ("USDCADm");}
      if (Trade_USDCAD && aCAD-aUSD<=CloseTrigger && aUSD>0 && aCAD>0) {CloseSellsNOW("USDCADm");}
      if (Trade_EURJPY && aEUR-aJPY<=CloseTrigger && aEUR>0 && aJPY>0) {CloseBuysNOW ("EURJPYm");}
      if (Trade_EURJPY && aJPY-aEUR<=CloseTrigger && aEUR>0 && aJPY>0) {CloseSellsNOW("EURJPYm");}
      if (Trade_EURGBP && aEUR-aGBP<=CloseTrigger && aEUR>0 && aGBP>0) {CloseBuysNOW ("EURGBPm");}
      if (Trade_EURGBP && aGBP-aEUR<=CloseTrigger && aEUR>0 && aGBP>0) {CloseSellsNOW("EURGBPm");}
      if (Trade_EURCHF && aEUR-aCHF<=CloseTrigger && aEUR>0 && aCHF>0) {CloseBuysNOW ("EURCHFm");}
      if (Trade_EURCHF && aCHF-aEUR<=CloseTrigger && aEUR>0 && aCHF>0) {CloseSellsNOW("EURCHFm");}
      if (Trade_EURAUD && aEUR-aAUD<=CloseTrigger && aEUR>0 && aAUD>0) {CloseBuysNOW ("EURAUDm");}
      if (Trade_EURAUD && aAUD-aEUR<=CloseTrigger && aEUR>0 && aAUD>0) {CloseSellsNOW("EURAUDm");}
      if (Trade_GBPJPY && aGBP-aJPY<=CloseTrigger && aGBP>0 && aJPY>0) {CloseBuysNOW ("GBPUSDm");}
      if (Trade_GBPJPY && aJPY-aGBP<=CloseTrigger && aGBP>0 && aJPY>0) {CloseSellsNOW("GBPUSDm");}
      if (Trade_GBPCHF && aGBP-aCHF<=CloseTrigger && aGBP>0 && aCHF>0) {CloseBuysNOW ("GBPCHFm");}
      if (Trade_GBPCHF && aCHF-aGBP<=CloseTrigger && aGBP>0 && aCHF>0) {CloseSellsNOW("GBPCHFm");}
   }
   else
   {
      //If xMeter reaches TradeTrigger then enter the trade
      //If xMeter falls to RecoverTrigger then allow another trade in same currency pair.
      if ( Trade_EURUSD && bEUR>0 && bUSD>0 && (bEUR-bUSD>=TradeTrigger || (aEUR-aUSD<=RecoverTrigger && GlobalVariableGet(a+"xMeter_EURUSDbuy" )==2))) {ManageBuy ("EURUSD",MaxSpread_EURUSD);}
      if ( Trade_EURUSD && bEUR>0 && bUSD>0 && (bUSD-bEUR>=TradeTrigger || (aUSD-aEUR<=RecoverTrigger && GlobalVariableGet(a+"xMeter_EURUSDsell")==2))) {ManageSell("EURUSD",MaxSpread_EURUSD);}
      if ( Trade_GBPUSD && bGBP>0 && bUSD>0 && (bGBP-bUSD>=TradeTrigger || (aGBP-aUSD<=RecoverTrigger && GlobalVariableGet(a+"xMeter_GBPUSDbuy" )==2))) {ManageBuy ("GBPUSD",MaxSpread_GBPUSD);}
      if ( Trade_GBPUSD && bGBP>0 && bUSD>0 && (bUSD-bGBP>=TradeTrigger || (aUSD-aGBP<=RecoverTrigger && GlobalVariableGet(a+"xMeter_GBPUSDsell")==2))) {ManageSell("GBPUSD",MaxSpread_GBPUSD);}
      if ( Trade_AUDUSD && bAUD>0 && bUSD>0 && (bAUD-bUSD>=TradeTrigger || (aAUD-aUSD<=RecoverTrigger && GlobalVariableGet(a+"xMeter_AUDUSDbuy" )==2))) {ManageBuy ("AUDUSD",MaxSpread_AUDUSD);}
      if ( Trade_AUDUSD && bAUD>0 && bUSD>0 && (bUSD-bAUD>=TradeTrigger || (aUSD-aAUD<=RecoverTrigger && GlobalVariableGet(a+"xMeter_AUDUSDsell")==2))) {ManageSell("AUDUSD",MaxSpread_AUDUSD);}
      if ( Trade_USDJPY && bUSD>0 && bJPY>0 && (bUSD-bJPY>=TradeTrigger || (aUSD-aJPY<=RecoverTrigger && GlobalVariableGet(a+"xMeter_USDJPYbuy" )==2))) {ManageBuy ("USDJPY",MaxSpread_USDJPY);}
      if ( Trade_USDJPY && bUSD>0 && bJPY>0 && (bJPY-bUSD>=TradeTrigger || (aJPY-aUSD<=RecoverTrigger && GlobalVariableGet(a+"xMeter_USDJPYsell")==2))) {ManageSell("USDJPY",MaxSpread_USDJPY);}
      if ( Trade_USDCHF && bUSD>0 && bCHF>0 && (bUSD-bCHF>=TradeTrigger || (aUSD-aCHF<=RecoverTrigger && GlobalVariableGet(a+"xMeter_USDCHFbuy" )==2))) {ManageBuy ("USDCHF",MaxSpread_USDCHF);}
      if ( Trade_USDCHF && bUSD>0 && bCHF>0 && (bCHF-bUSD>=TradeTrigger || (aCHF-aUSD<=RecoverTrigger && GlobalVariableGet(a+"xMeter_USDCHFsell")==2))) {ManageSell("USDCHF",MaxSpread_USDCHF);}
      if ( Trade_USDCAD && bUSD>0 && bCAD>0 && (bUSD-bCAD>=TradeTrigger || (aUSD-aCAD<=RecoverTrigger && GlobalVariableGet(a+"xMeter_USDCADbuy" )==2))) {ManageBuy ("USDCAD",MaxSpread_USDCAD);}
      if ( Trade_USDCAD && bUSD>0 && bCAD>0 && (bCAD-bUSD>=TradeTrigger || (aCAD-aUSD<=RecoverTrigger && GlobalVariableGet(a+"xMeter_USDCADsell")==2))) {ManageSell("USDCAD",MaxSpread_USDCAD);}
      if ( Trade_EURJPY && bEUR>0 && bJPY>0 && (bEUR-bJPY>=TradeTrigger || (aEUR-aJPY<=RecoverTrigger && GlobalVariableGet(a+"xMeter_EURJPYbuy" )==2))) {ManageBuy ("EURJPY",MaxSpread_EURJPY);}
      if ( Trade_EURJPY && bEUR>0 && bJPY>0 && (bJPY-bEUR>=TradeTrigger || (aJPY-aEUR<=RecoverTrigger && GlobalVariableGet(a+"xMeter_EURJPYsell")==2))) {ManageSell("EURJPY",MaxSpread_EURJPY);}
      if ( Trade_EURGBP && bEUR>0 && bGBP>0 && (bEUR-bGBP>=TradeTrigger || (aEUR-aGBP<=RecoverTrigger && GlobalVariableGet(a+"xMeter_EURGBPbuy" )==2))) {ManageBuy ("EURGBP",MaxSpread_EURGBP);}
      if ( Trade_EURGBP && bEUR>0 && bGBP>0 && (bGBP-bEUR>=TradeTrigger || (aGBP-aEUR<=RecoverTrigger && GlobalVariableGet(a+"xMeter_EURGBPsell")==2))) {ManageSell("EURGBP",MaxSpread_EURGBP);}
      if ( Trade_EURCHF && bEUR>0 && bCHF>0 && (bEUR-bCHF>=TradeTrigger || (aEUR-aCHF<=RecoverTrigger && GlobalVariableGet(a+"xMeter_EURCHFbuy" )==2))) {ManageBuy ("EURCHF",MaxSpread_EURCHF);}
      if ( Trade_EURCHF && bEUR>0 && bCHF>0 && (bCHF-bEUR>=TradeTrigger || (aCHF-aEUR<=RecoverTrigger && GlobalVariableGet(a+"xMeter_EURCHFsell")==2))) {ManageSell("EURCHF",MaxSpread_EURCHF);}
      if ( Trade_EURAUD && bEUR>0 && bAUD>0 && (bEUR-bAUD>=TradeTrigger || (aEUR-aAUD<=RecoverTrigger && GlobalVariableGet(a+"xMeter_EURAUDbuy" )==2))) {ManageBuy ("EURAUD",MaxSpread_EURAUD);}
      if ( Trade_EURAUD && bEUR>0 && bAUD>0 && (bAUD-bEUR>=TradeTrigger || (aAUD-aEUR<=RecoverTrigger && GlobalVariableGet(a+"xMeter_EURAUDsell")==2))) {ManageSell("EURAUD",MaxSpread_EURAUD);}
      if ( Trade_GBPJPY && bGBP>0 && bJPY>0 && (bGBP-bJPY>=TradeTrigger || (aGBP-aJPY<=RecoverTrigger && GlobalVariableGet(a+"xMeter_GBPJPYbuy" )==2))) {ManageBuy ("GBPJPY",MaxSpread_GBPJPY);}
      if ( Trade_GBPJPY && bGBP>0 && bJPY>0 && (bJPY-bGBP>=TradeTrigger || (aJPY-aGBP<=RecoverTrigger && GlobalVariableGet(a+"xMeter_GBPJPYsell")==2))) {ManageSell("GBPJPY",MaxSpread_GBPJPY);}
      if ( Trade_GBPCHF && bGBP>0 && bCHF>0 && (bGBP-bCHF>=TradeTrigger || (aGBP-aCHF<=RecoverTrigger && GlobalVariableGet(a+"xMeter_GBPCHFbuy" )==2))) {ManageBuy ("GBPCHF",MaxSpread_GBPCHF);}
      if ( Trade_GBPCHF && bGBP>0 && bCHF>0 && (bCHF-bGBP>=TradeTrigger || (aCHF-aGBP<=RecoverTrigger && GlobalVariableGet(a+"xMeter_GBPCHFsell")==2))) {ManageSell("GBPCHF",MaxSpread_GBPCHF);}
     
      //If xMeter falls to CloseTrigger then close all trades in that currency pair.
      if (Trade_EURUSD && aEUR-aUSD<=CloseTrigger && aEUR>0 && aUSD>0) {CloseBuysNOW ("EURUSD");}
      if (Trade_EURUSD && aUSD-aEUR<=CloseTrigger && aEUR>0 && aUSD>0) {CloseSellsNOW("EURUSD");}
      if (Trade_GBPUSD && aGBP-aUSD<=CloseTrigger && aGBP>0 && aUSD>0) {CloseBuysNOW ("GBPUSD");}
      if (Trade_GBPUSD && aUSD-aGBP<=CloseTrigger && aGBP>0 && aUSD>0) {CloseSellsNOW("GBPUSD");}
      if (Trade_AUDUSD && aAUD-aUSD<=CloseTrigger && aAUD>0 && aUSD>0) {CloseBuysNOW ("AUDUSD");}
      if (Trade_AUDUSD && aUSD-aAUD<=CloseTrigger && aAUD>0 && aUSD>0) {CloseSellsNOW("AUDUSD");}
      if (Trade_USDJPY && aUSD-aJPY<=CloseTrigger && aUSD>0 && aJPY>0) {CloseBuysNOW ("USDJPY");}
      if (Trade_USDJPY && aJPY-aUSD<=CloseTrigger && aUSD>0 && aJPY>0) {CloseSellsNOW("USDJPY");}
      if (Trade_USDCHF && aUSD-aCHF<=CloseTrigger && aUSD>0 && aCHF>0) {CloseBuysNOW ("USDCHF");}
      if (Trade_USDCHF && aCHF-aUSD<=CloseTrigger && aUSD>0 && aCHF>0) {CloseSellsNOW("USDCHF");}
      if (Trade_USDCAD && aUSD-aCAD<=CloseTrigger && aUSD>0 && aCAD>0) {CloseBuysNOW ("USDCAD");}
      if (Trade_USDCAD && aCAD-aUSD<=CloseTrigger && aUSD>0 && aCAD>0) {CloseSellsNOW("USDCAD");}
      if (Trade_EURJPY && aEUR-aJPY<=CloseTrigger && aEUR>0 && aJPY>0) {CloseBuysNOW ("EURJPY");}
      if (Trade_EURJPY && aJPY-aEUR<=CloseTrigger && aEUR>0 && aJPY>0) {CloseSellsNOW("EURJPY");}
      if (Trade_EURGBP && aEUR-aGBP<=CloseTrigger && aEUR>0 && aGBP>0) {CloseBuysNOW ("EURGBP");}
      if (Trade_EURGBP && aGBP-aEUR<=CloseTrigger && aEUR>0 && aGBP>0) {CloseSellsNOW("EURGBP");}
      if (Trade_EURCHF && aEUR-aCHF<=CloseTrigger && aEUR>0 && aCHF>0) {CloseBuysNOW ("EURCHF");}
      if (Trade_EURCHF && aCHF-aEUR<=CloseTrigger && aEUR>0 && aCHF>0) {CloseSellsNOW("EURCHF");}
      if (Trade_EURAUD && aEUR-aAUD<=CloseTrigger && aEUR>0 && aAUD>0) {CloseBuysNOW ("EURAUD");}
      if (Trade_EURAUD && aAUD-aEUR<=CloseTrigger && aEUR>0 && aAUD>0) {CloseSellsNOW("EURAUD");}
      if (Trade_GBPJPY && aGBP-aJPY<=CloseTrigger && aGBP>0 && aJPY>0) {CloseBuysNOW ("GBPUSD");}
      if (Trade_GBPJPY && aJPY-aGBP<=CloseTrigger && aGBP>0 && aJPY>0) {CloseSellsNOW("GBPUSD");}
      if (Trade_GBPCHF && aGBP-aCHF<=CloseTrigger && aGBP>0 && aCHF>0) {CloseBuysNOW ("GBPCHF");}
      if (Trade_GBPCHF && aCHF-aGBP<=CloseTrigger && aGBP>0 && aCHF>0) {CloseSellsNOW("GBPCHF");}
   }
   Comments();
   paintAlert();
}//---- int start()
  
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
void CalculateLotPrecision()
{
   double lotstep=MarketInfo(Symbol(),MODE_LOTSTEP);
   if(lotstep==1) lotPrecision=0;
   if(lotstep==0.1) lotPrecision=1;
   if(lotstep==0.01) lotPrecision=2;
   if(lotstep==0.001) lotPrecision=3;
}

//------------------------------------------------------------------------------------------------
double AutoLot()
{
   double lot;
   lot=NormalizeDouble(AccountBalance()/10000,lotPrecision)*AccountRiskPercent;
   if(MicroAccount)lot=NormalizeDouble(lot/10,lotPrecision);
   if(lot<MarketInfo(Symbol(),MODE_MINLOT))lot=MarketInfo(Symbol(),MODE_MINLOT);
   return(lot);
}

//+------------------------------------------------------------------+
void ManageBuy(string symb, int spd)//Opens trade if xMeter currency difference is greater than TradeTrigger.
{
   double ask   = MarketInfo(symb,MODE_ASK);
   double point = MarketInfo(symb,MODE_POINT);
   double sprd  = MarketInfo(symb,MODE_SPREAD);
   bool   BuyOK = false;
   if( sprd <= spd && Initialized )
   {         
      int buyorders=0, sellorders=0, BuyProfits=0;
      double lt, LowestBuy = 1000;
      for(int j=0;j<OrdersTotal();j++)
      {
         OrderSelect(j, SELECT_BY_POS, MODE_TRADES);
         if(OrderMagicNumber()==MagicNumber && OrderSymbol()==symb && OrderType()==OP_SELL) sellorders++;
         if(OrderMagicNumber()==MagicNumber && OrderSymbol()==symb && OrderType()==OP_BUY) 
         {
            buyorders++; 
            if (OrderOpenPrice() < LowestBuy) LowestBuy = OrderOpenPrice();
         }
      }                                          
      double sdv = sprd*point*3+MathPow(iStdDev(symb,15,6,0,0,0,0),0.9);
      if(sdv<20*MarketInfo(symb,MODE_POINT))sdv=20*MarketInfo(symb,MODE_POINT);//Set minimum of 20 for grid spacing
      if (!PauseTrading && buyorders==0 && GlobalVariableGet(a+"xMeter_"+symb+"buy")==1)    //---- we can go trading
      {
         lt=InitialLotSize;   
         if((1-(AccountFreeMarginCheck(symb,OP_BUY,lt)/AccountBalance()))*100>15){return(0);}//Do not open new cycle of trades if current trades are using more than 15% margin.  The other 85% may be needed to ride out current loss.
         BuyOK=true;
      }
      else   
      if (buyorders>0 && ask<=LowestBuy-sdv && GlobalVariableGet(a+"xMeter_"+symb+"buy")==2)//---- we can go trading
      {
         if(buyorders>3)lt=NormalizeDouble(MathPow(1.5,buyorders-2)*InitialLotSize, lotPrecision);
         else lt=NormalizeDouble((InitialLotSize),lotPrecision);
         BuyOK=true;
      }   
      if(!BuyOK)return(0);
      buyorders++;
      int Retry=0, ticket=0; while (Retry<5 && !IsTradeAllowed()) {Retry++; Sleep(2000); }
      ticket=OrderSend(symb,OP_BUY,lt,ask,Slippage,0,0,EAversion+symb+" #"+buyorders,MagicNumber,0,Blue);Sleep(1000);
      if( ticket > 0 )
      {
         if(OrderSelect(ticket,SELECT_BY_TICKET,MODE_TRADES))
         {
            GlobalVariableSet(a+"xMeter_"+symb+"buy",2);
            Print("BUY order opened: ",symb," at ",OrderOpenPrice());
         }
      }
      else Print("Error opening ",symb," BUY order - Error#: ",GetLastError());
   }else Print("Error opening ",symb," BUY order - Error#: BigSpread");
}
   
//+------------------------------------------------------------------+
void ManageSell(string symb, int spd)//Opens trade if xMeter currency difference is greater than TradeTrigger.
{
   double bid   = MarketInfo(symb,MODE_BID);
   double point = MarketInfo(symb,MODE_POINT);
   double sprd  = MarketInfo(symb,MODE_SPREAD);
   bool   SellOK= false;
   if( sprd <= spd  && Initialized)
   {
      int buyorders=0, sellorders=0, SellProfits=0;
      double lt, HighestSell=0;
      for(int j=0;j<OrdersTotal();j++)
      {
         OrderSelect(j, SELECT_BY_POS, MODE_TRADES);
         if(OrderMagicNumber()==MagicNumber && OrderSymbol()==symb && OrderType()==OP_BUY) buyorders++;
         if(OrderMagicNumber()==MagicNumber && OrderSymbol()==symb && OrderType()==OP_SELL) 
         {
            sellorders++; 
            if (OrderOpenPrice() > HighestSell) HighestSell = OrderOpenPrice();
         }
      }                                          
      double sdv = sprd*point*3+MathPow(iStdDev(symb,15,6,0,0,0,0),0.9);
      if(sdv<20*MarketInfo(symb,MODE_POINT))sdv=20*MarketInfo(symb,MODE_POINT);//Set minimum of 20 for grid spacing
      if (!PauseTrading && sellorders==0 && GlobalVariableGet(a+"xMeter_"+symb+"sell")==1)//---- we can go trading
      {
         lt=InitialLotSize;
         if((1-(AccountFreeMarginCheck(symb,OP_SELL,lt)/AccountBalance()))*100>15){return(0);}//Do not open new cycle of trades if current trades are using more than 15% margin.  The other 85% may be needed to ride out current loss.
         SellOK=true;
      }    
      else  
      if (sellorders>0 && bid>=HighestSell+sdv && GlobalVariableGet(a+"xMeter_"+symb+"sell")==2)//---- we can go trading
      {
         if(sellorders>3)lt=NormalizeDouble(MathPow(1.5,sellorders-2)*InitialLotSize,lotPrecision);
         else lt=NormalizeDouble((InitialLotSize),lotPrecision);
         SellOK=true;
      }
      if(!SellOK)return(0);
      sellorders++;
      int Retry=0, ticket=0; while (Retry<5 && !IsTradeAllowed()) {Retry++; Sleep(2000); }
      ticket=OrderSend(symb,OP_SELL,lt,bid,Slippage,0,0,EAversion+symb+" #"+sellorders,MagicNumber,0,Red);Sleep(1000);
      if( ticket > 0 )
      {
         if(OrderSelect(ticket,SELECT_BY_TICKET,MODE_TRADES))
         {
            GlobalVariableSet(a+"xMeter_"+symb+"sell",2);
            Print("SELL order opened : ",symb," at ",OrderOpenPrice());
         }
      }
      else Print("Error opening ",symb," SELL order : ",GetLastError()); 
   }else Print("Error opening ",symb," SELL order - Error#: BigSpread");
}

//+------------------------------------------------------------------+
void CloseBuysNOW(string symb)//Closes trade if xMeter currency difference is less than CloseTarget.
{      
   for(int j=0;j<OrdersTotal();j++)
   {
      OrderSelect(j, SELECT_BY_POS, MODE_TRADES);      
      if(OrderMagicNumber()==MagicNumber && OrderSymbol()==symb && OrderType()==OP_BUY && Initialized) 
      {
         int Retry=0, ticket=0; while (Retry<5 && !IsTradeAllowed()) {Retry++; Sleep(2000); }
         ticket=OrderClose(OrderTicket(),OrderLots(),OrderClosePrice(),Slippage,Purple);
         if( ticket > 0 )
         {
            if(OrderSelect(ticket,SELECT_BY_TICKET,MODE_TRADES))
            {
               Print("BUY order closed : ",symb," at ",OrderClosePrice());
            }
         }
         else Print("Error closing ",symb," BUY order : ",GetLastError()); 
      }   
   } 
   int buyorders=0;
   for(j=0;j<OrdersTotal();j++)//Check if any Orders didn't close because of requotes.
   {
      OrderSelect(j, SELECT_BY_POS, MODE_TRADES);
      if(OrderMagicNumber()==MagicNumber && OrderSymbol()==symb && OrderType()==OP_BUY) buyorders++;
      if(buyorders>0)CloseBuysNOW(symb);
   }   
}                                        

//+------------------------------------------------------------------+
void CloseSellsNOW(string symb)//Closes trade if xMeter currency difference is less than CloseTarget.
{
   for(int j=0;j<OrdersTotal();j++)
   {
      OrderSelect(j, SELECT_BY_POS, MODE_TRADES);
      if(OrderMagicNumber()==MagicNumber && OrderSymbol()==symb && OrderType()==OP_SELL && Initialized) 
      {
         int Retry=0, ticket=0; while (Retry<5 && !IsTradeAllowed()) {Retry++; Sleep(2000); }
         ticket=OrderClose(OrderTicket(),OrderLots(),OrderClosePrice(),Slippage,Purple);
         if( ticket > 0 )
         {
            if(OrderSelect(ticket,SELECT_BY_TICKET,MODE_TRADES))
            {
               Print("SELL order closed : ",symb," at ",OrderClosePrice());
            }
         }
         else Print("Error closing ",symb," SELL order : ",GetLastError()); 
      }   
   } 
   int sellorders=0;       
   for(j=0;j<OrdersTotal();j++)//Check if any Orders didn't close because of requotes.
   {
      OrderSelect(j, SELECT_BY_POS, MODE_TRADES);
      if(OrderMagicNumber()==MagicNumber && OrderSymbol()==symb && OrderType()==OP_SELL) sellorders++;
      if(sellorders>0)CloseSellsNOW(symb);
   }   
}   

//+------------------------------------------------------------------+
void ExitAllTradesNOW()//Close all trades made by EA if at breakeven and in recovery mode
{
   bool success;
   for (int cnt = OrdersTotal() - 1; cnt >= 0; cnt --)
   {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if (OrderMagicNumber()==MagicNumber && Initialized)
      {
         int Retry=0; while (Retry<5 && !IsTradeAllowed()) { Retry++; Sleep(2000); }
         success=OrderClose(OrderTicket(), OrderLots(), OrderClosePrice(), Slippage, Lime);
         if(success==true){if(!IsTesting())   Print("Closed all positions");}
      } 
   }
   int Orders=0;
   for(cnt=0;cnt<OrdersTotal();cnt++)//Check if any Orders didn't close because of requotes.
   {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if(OrderMagicNumber()==MagicNumber) Orders++;
      if(Orders>0)ExitAllTradesNOW();
   }  
}   

//+------------------------------------------------------------------+
void DynamicBuySL(string symb, int buyorders, double ask, double bid, double point, int sprd)//If trade is in profit greater than breakeven then set breakeven stop or adjust DynamicStop.
{     
   for(int cnt=0;cnt<OrdersTotal();cnt++)
   {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if(OrderMagicNumber()==MagicNumber && OrderSymbol()==symb && buyorders==1)
      {
         double sdv=(sprd*point*3)+iStdDev(symb,15,6,0,0,0,0)*1.5;//Use Spread+StdDev to set SL 
         if(sdv<25*point)sdv=25*point;//Set minimum of 25 for trailing stop
         double Breakeven=sdv/1.5;
         
         if(OrderType()==OP_BUY)
         {      
            if(bid-OrderOpenPrice()>=Breakeven && OrderStopLoss()<OrderOpenPrice())
            OrderModify(OrderTicket(),OrderOpenPrice(),OrderOpenPrice(),OrderTakeProfit(),0,Red);
            
            sl=bid-sdv;
            if(sl-OrderOpenPrice()>0)
            {
               if(sl-OrderStopLoss()>5*point)//Adjust in 5 pip increments
               OrderModify(OrderTicket(),OrderOpenPrice(),sl,OrderTakeProfit(),0,Red);
            }   
         }
      }
   }
}

//+------------------------------------------------------------------+
void DynamicSellSL(string symb, int sellorders, double ask, double bid, double point, int sprd)//If trade is in profit greater than breakeven then set breakeven stop or adjust DynamicStop.
{     
   for(int cnt=0;cnt<OrdersTotal();cnt++)
   {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if(OrderMagicNumber()==MagicNumber && OrderSymbol()==symb && sellorders==1)
      {
         double sdv=(sprd*point*3)+iStdDev(symb,15,6,0,0,0,0)*1.5;//Use Spread+StdDev to set SL 
         if(sdv<25*point)sdv=25*point;//Set minimum of 25 for trailing stop
         double Breakeven=sdv/1.5;
         
         if(OrderType()==OP_SELL)  
         {    
            if(OrderOpenPrice()-ask>=Breakeven && (OrderStopLoss()>OrderOpenPrice() || OrderStopLoss()==0))
            OrderModify(OrderTicket(),OrderOpenPrice(),OrderOpenPrice(),OrderTakeProfit(),0,Red);
            
            sl=ask+sdv;
            if(OrderOpenPrice()-sl>0)
            {
               if(OrderStopLoss()-sl>5*point)//Adjust in 5 pip increments
               OrderModify(OrderTicket(),OrderOpenPrice(),sl,OrderTakeProfit(),0,Red);
            }   
         }
      }
   }
}

//+------------------------------------------------------------------+
void DynamicBuyTP(string symb, int buyorders, double ask, double bid, double point, int sprd)//Adjust takeprofit as three times the Standard Deviation
{
   for(int cnt=0;cnt<OrdersTotal();cnt++)
   {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if(OrderMagicNumber()==MagicNumber && OrderSymbol()==symb)
      {      
         double sdv=(sprd*point*3)+iStdDev(OrderSymbol(),15,6,0,0,0,0)*3;//Use Spread+StdDev to set TP
         if(sdv<30*point)sdv=30*point;
         if(OrderType()==OP_BUY)
         {
            if(buyorders==1)
            {   
               tp=OrderOpenPrice()+sdv;
               if(MathAbs(OrderTakeProfit()-tp)>5*point && bid>OrderOpenPrice())//Adjust in 5 pip increments
               OrderModify(OrderTicket(),OrderOpenPrice(),OrderStopLoss(),tp,0,Blue);
            }   
            if(buyorders>1)
            {
               if(bid>OrderOpenPrice() && OrderTakeProfit()>0)OrderModify(OrderTicket(),OrderOpenPrice(),OrderStopLoss(),0,0,CLR_NONE);//If in recovery mode then use profit averaging instead of TP
            }
         }   
      }
   }
}
   
//+------------------------------------------------------------------+
void DynamicSellTP(string symb, int sellorders, double ask, double bid, double point, int sprd)//Adjust takeprofit as three times the Standard Deviation
{
   for(int cnt=0;cnt<OrdersTotal();cnt++)
   {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if(OrderMagicNumber()==MagicNumber && OrderSymbol()==symb)
      {      
         double sdv=(sprd*point*3)+iStdDev(OrderSymbol(),15,6,0,0,0,0)*3;//Use Spread+StdDev to set TP
         if(sdv<30*point)sdv=30*point;
         if(OrderType()==OP_SELL)
         {
            if(sellorders==1)
            {
               tp=OrderOpenPrice()-sdv;
               if(MathAbs(OrderTakeProfit()-tp)>5*point && ask<OrderOpenPrice())//Adjust in 5 pip increments
               OrderModify(OrderTicket(),OrderOpenPrice(),OrderStopLoss(),tp,0,Blue);
            }   
            if(sellorders>1)
            {
               if(ask<OrderOpenPrice() && OrderTakeProfit()>0)OrderModify(OrderTicket(),OrderOpenPrice(),OrderStopLoss(),0,0,CLR_NONE);//If in recovery mode then use profit averaging instead of TP
            }
         }   
      }
   }
}
   
//+------------------------------------------------------------------+
void AdjustTemporarySLTP() //Incase internet disconnect only! SL moves away from open price but not closer.  If breakeven stop is already set then it does not adjust SL.
{
   for (int cnt=0;cnt<OrdersTotal();cnt++)
   {  
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      {
         if (OrderMagicNumber()==MagicNumber && !IsTesting())
         {
            if (OrderType()==OP_BUY)
            {
               tempSL=MarketInfo(OrderSymbol(),MODE_BID) - 150*MarketInfo(OrderSymbol(),MODE_POINT);
               if((OrderStopLoss()-tempSL>10*MarketInfo(OrderSymbol(),MODE_POINT) || OrderStopLoss()==0) && OrderStopLoss()<OrderOpenPrice())//Adjust in 10 pip increments away from current price incase of internet disconnect only!
               OrderModify(OrderTicket(),OrderOpenPrice(),tempSL,OrderTakeProfit(),0,CLR_NONE);
          
               if(MarketInfo(OrderSymbol(),MODE_BID)<OrderOpenPrice())tempTP=OrderOpenPrice()+30*MarketInfo(OrderSymbol(),MODE_POINT); else tempTP=0;//TempTP incase internet disconnect only! Once price is above breakeven then TP will switch to DynamicTP.
               if(MathAbs(OrderTakeProfit()-tempTP)>0 && tempTP>0)
               OrderModify(OrderTicket(),OrderOpenPrice(),OrderStopLoss(),tempTP,0,Blue);
            }
            if (OrderType()==OP_SELL)
            {
               tempSL=MarketInfo(OrderSymbol(),MODE_ASK) + 150*MarketInfo(OrderSymbol(),MODE_POINT);
               if(tempSL-OrderStopLoss()>10*MarketInfo(OrderSymbol(),MODE_POINT)  && (OrderStopLoss()>OrderOpenPrice() || OrderStopLoss()==0))//Adjust in 10 pip increments away from current price incase of internet disconnect only!
               OrderModify(OrderTicket(),OrderOpenPrice(),tempSL,OrderTakeProfit(),0,CLR_NONE);

               if(MarketInfo(OrderSymbol(),MODE_ASK)>OrderOpenPrice())tempTP=OrderOpenPrice()-30*MarketInfo(OrderSymbol(),MODE_POINT); else tempTP=0;//TempTP incase internet disconnect only! Once price is above breakeven then TP will switch to DynamicTP.
               if(MathAbs(OrderTakeProfit()-tempTP)>0 && tempTP>0) 
               OrderModify(OrderTicket(),OrderOpenPrice(),OrderStopLoss(),tempTP,0,Blue);
            }   
         }
      }
   }
} 

//+------------------------------------------------------------------+
int PauseAtTime(int startTime, int endTime)//Will pause trading between startime and endtime. (DayOfWeek,Hour,Hour,Minute,Minute)
{
   if(startTime!=0)
   {
      int t=DayOfWeek()*10000+TimeHour(TimeCurrent())*100+TimeMinute(TimeCurrent());
      if(t>=startTime || t<=endTime)PauseTrading=true;else PauseTrading=false;//Allow current cycle of trades to continue until all are closed by EA then pause
   }
}

//+------------------------------------------------------------------+
void Comments()
{
   if (Initialized && IsConnected())
   {   
      if(AlertNOW)Alert2="EA has Resumed Trading";
      else Alert2="";
      if(TimeCurrent()>LastErrorTime+60)AlertNOW=false;
      Time1="Brokers Time is "+TimeToStr(TimeCurrent(), TIME_SECONDS);
      message="";     
   }
   else
   if(IsConnected()) 
   {
      dot=Volume[0]-tick;if(dot<=1)dots="."; if(dot==2)dots=". ."; if(dot==3)dots=". . ."; if(dot==4)dots=". . . ."; if(dot==5)dots=". . . . ."; if(dot==6)dots=". . . . . .";if(dot==7)dots=". . . . . . .";if(dot==8)dots=". . . . . . . .";if(dot==9)dots=". . . . . . . . .";if(dot>=10)dots=". . . . . . . . . ."; 
      Alert2=dots+"INITIALIZING EXPERT";
      message="";
   }   
   Comment(message);
}
  
//================================================================================================================================
//---- Display the !xMeter's values
void initGraph()
  {
   objectDeleteAll();
   
   objectCreate("usd_1",130,43);
   objectCreate("usd_2",130,35);
   objectCreate("usd_3",130,27);
   objectCreate("usd_4",130,19);
   objectCreate("usd_5",130,11);
   objectCreate("usd",132,12,"USD",7,"Arial Narrow",SkyBlue);
   objectCreate("usdp",134,21,DoubleToStr(9,1),8,"Arial Narrow",Silver);
   
   objectCreate("eur_1",110,43);
   objectCreate("eur_2",110,35);
   objectCreate("eur_3",110,27);
   objectCreate("eur_4",110,19);
   objectCreate("eur_5",110,11);
   objectCreate("eur",112,12,"EUR",7,"Arial Narrow",SkyBlue);
   objectCreate("eurp",114,21,DoubleToStr(9,1),8,"Arial Narrow",Silver);
   
   objectCreate("gbp_1",90,43);
   objectCreate("gbp_2",90,35);
   objectCreate("gbp_3",90,27);
   objectCreate("gbp_4",90,19);
   objectCreate("gbp_5",90,11);
   objectCreate("gbp",92,12,"GBP",7,"Arial Narrow",SkyBlue);
   objectCreate("gbpp",94,21,DoubleToStr(9,1),8,"Arial Narrow",Silver);
   
   objectCreate("chf_1",70,43);
   objectCreate("chf_2",70,35);
   objectCreate("chf_3",70,27);
   objectCreate("chf_4",70,19);
   objectCreate("chf_5",70,11);
   objectCreate("chf",72,12,"CHF",7,"Arial Narrow",SkyBlue);
   objectCreate("chfp",74,21,DoubleToStr(9,1),8,"Arial Narrow",Silver);

   objectCreate("cad_1",50,43);
   objectCreate("cad_2",50,35);   
   objectCreate("cad_3",50,27);
   objectCreate("cad_4",50,19);
   objectCreate("cad_5",50,11);
   objectCreate("cad",52,12,"CAD",7,"Arial Narrow",SkyBlue);
   objectCreate("cadp",54,21,DoubleToStr(9,1),8,"Arial Narrow",Silver);
   
   objectCreate("aud_1",30,43);
   objectCreate("aud_2",30,35);
   objectCreate("aud_3",30,27);
   objectCreate("aud_4",30,19);
   objectCreate("aud_5",30,11);
   objectCreate("aud",32,12,"AUD",7,"Arial Narrow",SkyBlue);
   objectCreate("audp",34,21,DoubleToStr(9,1),8,"Arial Narrow",Silver);

   objectCreate("jpy_1",10,43);
   objectCreate("jpy_2",10,35);
   objectCreate("jpy_3",10,27);
   objectCreate("jpy_4",10,19);
   objectCreate("jpy_5",10,11);
   objectCreate("jpy",13,12,"JPY",7,"Arial Narrow",SkyBlue);
   objectCreate("jpyp",14,21,DoubleToStr(9,1),8,"Arial Narrow",Silver);
   
   objectCreate("line",10,6,"-----------------------------------",10,"Arial",DimGray);  
   objectCreate("line1",10,27,"-----------------------------------",10,"Arial",DimGray);  
   objectCreate("line2",10,69,"-----------------------------------",10,"Arial",DimGray);
   objectCreate("sign",11,1," Price Meter System GPL ",8,"Arial Narrow",DimGray);
   objectCreate("Alert2",11,95,Alert2,10,"Arial",Red);
   objectCreate("time",11,75,Time1,8,"Arial",Yellow);
   WindowRedraw();    
  }

void objectCreate(string name,int x,int y,string text="-",int size=42,
                  string font="Arial",color colour=CLR_NONE)
  {
   ObjectCreate(name,OBJ_LABEL,0,0,0);
   ObjectSet(name,OBJPROP_CORNER,3);
   ObjectSet(name,OBJPROP_COLOR,colour);
   ObjectSet(name,OBJPROP_XDISTANCE,x);
   ObjectSet(name,OBJPROP_YDISTANCE,y);
   ObjectSetText(name,text,size,font,colour);
  }

void objectBlank()
  {
   ObjectSet("usd_1",OBJPROP_COLOR,CLR_NONE);
   ObjectSet("usd_2",OBJPROP_COLOR,CLR_NONE);
   ObjectSet("usd_3",OBJPROP_COLOR,CLR_NONE);
   ObjectSet("usd_4",OBJPROP_COLOR,CLR_NONE);
   ObjectSet("usd_5",OBJPROP_COLOR,CLR_NONE);
//   ObjectSet("usd",OBJPROP_COLOR,CLR_NONE);
   ObjectSet("usdp",OBJPROP_COLOR,CLR_NONE);

   ObjectSet("eur_1",OBJPROP_COLOR,CLR_NONE);
   ObjectSet("eur_2",OBJPROP_COLOR,CLR_NONE);
   ObjectSet("eur_3",OBJPROP_COLOR,CLR_NONE);
   ObjectSet("eur_4",OBJPROP_COLOR,CLR_NONE);
   ObjectSet("eur_5",OBJPROP_COLOR,CLR_NONE);
//   ObjectSet("eur",OBJPROP_COLOR,CLR_NONE);
   ObjectSet("eurp",OBJPROP_COLOR,CLR_NONE);

   ObjectSet("gbp_1",OBJPROP_COLOR,CLR_NONE);
   ObjectSet("gbp_2",OBJPROP_COLOR,CLR_NONE);
   ObjectSet("gbp_3",OBJPROP_COLOR,CLR_NONE);
   ObjectSet("gbp_4",OBJPROP_COLOR,CLR_NONE);
   ObjectSet("gbp_5",OBJPROP_COLOR,CLR_NONE);
//   ObjectSet("gbp",OBJPROP_COLOR,CLR_NONE);
   ObjectSet("gbpp",OBJPROP_COLOR,CLR_NONE);

   ObjectSet("chf_1",OBJPROP_COLOR,CLR_NONE);
   ObjectSet("chf_2",OBJPROP_COLOR,CLR_NONE);
   ObjectSet("chf_3",OBJPROP_COLOR,CLR_NONE);
   ObjectSet("chf_4",OBJPROP_COLOR,CLR_NONE);
   ObjectSet("chf_5",OBJPROP_COLOR,CLR_NONE);
//   ObjectSet("chf",OBJPROP_COLOR,CLR_NONE);
   ObjectSet("chfp",OBJPROP_COLOR,CLR_NONE);

   ObjectSet("cad_1",OBJPROP_COLOR,CLR_NONE);
   ObjectSet("cad_2",OBJPROP_COLOR,CLR_NONE);
   ObjectSet("cad_3",OBJPROP_COLOR,CLR_NONE);
   ObjectSet("cad_4",OBJPROP_COLOR,CLR_NONE);
   ObjectSet("cad_5",OBJPROP_COLOR,CLR_NONE);
//   ObjectSet("cad",OBJPROP_COLOR,CLR_NONE);
   ObjectSet("cadp",OBJPROP_COLOR,CLR_NONE);

   ObjectSet("aud_1",OBJPROP_COLOR,CLR_NONE);
   ObjectSet("aud_2",OBJPROP_COLOR,CLR_NONE);
   ObjectSet("aud_3",OBJPROP_COLOR,CLR_NONE);
   ObjectSet("aud_4",OBJPROP_COLOR,CLR_NONE);
   ObjectSet("aud_5",OBJPROP_COLOR,CLR_NONE);
//   ObjectSet("aud",OBJPROP_COLOR,CLR_NONE);
   ObjectSet("audp",OBJPROP_COLOR,CLR_NONE);

   ObjectSet("jpy_1",OBJPROP_COLOR,CLR_NONE);
   ObjectSet("jpy_2",OBJPROP_COLOR,CLR_NONE);
   ObjectSet("jpy_3",OBJPROP_COLOR,CLR_NONE);
   ObjectSet("jpy_4",OBJPROP_COLOR,CLR_NONE);
   ObjectSet("jpy_5",OBJPROP_COLOR,CLR_NONE);
//   ObjectSet("jpy",OBJPROP_COLOR,CLR_NONE);
   ObjectSet("jpyp",OBJPROP_COLOR,CLR_NONE);
   
//   ObjectSet("line1",OBJPROP_COLOR,CLR_NONE);
//   ObjectSet("line2",OBJPROP_COLOR,CLR_NONE); 
   ObjectSet("Alert2",OBJPROP_COLOR,CLR_NONE);
   ObjectSet("time",OBJPROP_COLOR,CLR_NONE);
  }
  
void objectDeleteAll()
{
   ObjectDelete("usd_1");
   ObjectDelete("usd_2");
   ObjectDelete("usd_3");
   ObjectDelete("usd_4");
   ObjectDelete("usd_5");
   ObjectDelete("usd");
   ObjectDelete("usdp");

   ObjectDelete("eur_1");
   ObjectDelete("eur_2");
   ObjectDelete("eur_3");
   ObjectDelete("eur_4");
   ObjectDelete("eur_5");
   ObjectDelete("eur");
   ObjectDelete("eurp");

   ObjectDelete("gbp_1");
   ObjectDelete("gbp_2");
   ObjectDelete("gbp_3");
   ObjectDelete("gbp_4");
   ObjectDelete("gbp_5");
   ObjectDelete("gbp");
   ObjectDelete("gbpp");

   ObjectDelete("chf_1");
   ObjectDelete("chf_2");
   ObjectDelete("chf_3");
   ObjectDelete("chf_4");
   ObjectDelete("chf_5");
   ObjectDelete("chf");
   ObjectDelete("chfp");

   ObjectDelete("cad_1");
   ObjectDelete("cad_2");
   ObjectDelete("cad_3");
   ObjectDelete("cad_4");
   ObjectDelete("cad_5");
   ObjectDelete("cad");
   ObjectDelete("cadp");

   ObjectDelete("aud_1");
   ObjectDelete("aud_2");
   ObjectDelete("aud_3");
   ObjectDelete("aud_4");
   ObjectDelete("aud_5");
   ObjectDelete("aud");
   ObjectDelete("audp");

   ObjectDelete("jpy_1");
   ObjectDelete("jpy_2");
   ObjectDelete("jpy_3");
   ObjectDelete("jpy_4");
   ObjectDelete("jpy_5");
   ObjectDelete("jpy");
   ObjectDelete("jpyp");
   
   ObjectDelete("line");  
   ObjectDelete("line1");  
   ObjectDelete("line2");
   ObjectDelete("sign");
   ObjectDelete("Alert2");
   ObjectDelete("time");

}
  
void paintUSD(double value)
  {
   if (value > 0) ObjectSet("usd_5",OBJPROP_COLOR,Red);
   if (value > 2) ObjectSet("usd_4",OBJPROP_COLOR,Orange);
   if (value > 4) ObjectSet("usd_3",OBJPROP_COLOR,Gold);   
   if (value > 6) ObjectSet("usd_2",OBJPROP_COLOR,YellowGreen);
   if (value > 7) ObjectSet("usd_1",OBJPROP_COLOR,Lime);
   ObjectSet("usd",OBJPROP_COLOR,SkyBlue);
   ObjectSetText("usdp",DoubleToStr(value,1),8,"Arial Narrow",Silver);
  }

void paintEUR(double value)
  {
   if (value > 0) ObjectSet("eur_5",OBJPROP_COLOR,Red);
   if (value > 2) ObjectSet("eur_4",OBJPROP_COLOR,Orange);
   if (value > 4) ObjectSet("eur_3",OBJPROP_COLOR,Gold);   
   if (value > 6) ObjectSet("eur_2",OBJPROP_COLOR,YellowGreen);
   if (value > 7) ObjectSet("eur_1",OBJPROP_COLOR,Lime);
   ObjectSet("eur",OBJPROP_COLOR,SkyBlue);
   ObjectSetText("eurp",DoubleToStr(value,1),8,"Arial Narrow",Silver);
  }

void paintGBP(double value)
  {
   if (value > 0) ObjectSet("gbp_5",OBJPROP_COLOR,Red);
   if (value > 2) ObjectSet("gbp_4",OBJPROP_COLOR,Orange);
   if (value > 4) ObjectSet("gbp_3",OBJPROP_COLOR,Gold);   
   if (value > 6) ObjectSet("gbp_2",OBJPROP_COLOR,YellowGreen);
   if (value > 7) ObjectSet("gbp_1",OBJPROP_COLOR,Lime);
   ObjectSet("gbp",OBJPROP_COLOR,SkyBlue);
   ObjectSetText("gbpp",DoubleToStr(value,1),8,"Arial Narrow",Silver);
  }

void paintCHF(double value)
  {
   if (value > 0) ObjectSet("chf_5",OBJPROP_COLOR,Red);
   if (value > 2) ObjectSet("chf_4",OBJPROP_COLOR,Orange);
   if (value > 4) ObjectSet("chf_3",OBJPROP_COLOR,Gold);   
   if (value > 6) ObjectSet("chf_2",OBJPROP_COLOR,YellowGreen);
   if (value > 7) ObjectSet("chf_1",OBJPROP_COLOR,Lime);
   ObjectSet("chf",OBJPROP_COLOR,SkyBlue);
   ObjectSetText("chfp",DoubleToStr(value,1),8,"Arial Narrow",Silver);
  }

void paintCAD(double value)
  {
   if (value > 0) ObjectSet("cad_5",OBJPROP_COLOR,Red);
   if (value > 2) ObjectSet("cad_4",OBJPROP_COLOR,Orange);
   if (value > 4) ObjectSet("cad_3",OBJPROP_COLOR,Gold);   
   if (value > 6) ObjectSet("cad_2",OBJPROP_COLOR,YellowGreen);
   if (value > 7) ObjectSet("cad_1",OBJPROP_COLOR,Lime);
   ObjectSet("cad",OBJPROP_COLOR,SkyBlue);
   ObjectSetText("cadp",DoubleToStr(value,1),8,"Arial Narrow",Silver);
  }

void paintAUD(double value)
  {
   if (value > 0) ObjectSet("aud_5",OBJPROP_COLOR,Red);
   if (value > 2) ObjectSet("aud_4",OBJPROP_COLOR,Orange);
   if (value > 4) ObjectSet("aud_3",OBJPROP_COLOR,Gold);   
   if (value > 6) ObjectSet("aud_2",OBJPROP_COLOR,YellowGreen);
   if (value > 7) ObjectSet("aud_1",OBJPROP_COLOR,Lime);
   ObjectSet("aud",OBJPROP_COLOR,SkyBlue);
   ObjectSetText("audp",DoubleToStr(value,1),8,"Arial Narrow",Silver);
  }

void paintJPY(double value)
  {
   if (value > 0) ObjectSet("jpy_5",OBJPROP_COLOR,Red);
   if (value > 2) ObjectSet("jpy_4",OBJPROP_COLOR,Orange);
   if (value > 4) ObjectSet("jpy_3",OBJPROP_COLOR,Gold);   
   if (value > 6) ObjectSet("jpy_2",OBJPROP_COLOR,YellowGreen);
   if (value > 7) ObjectSet("jpy_1",OBJPROP_COLOR,Lime);
   ObjectSet("jpy",OBJPROP_COLOR,SkyBlue);
   ObjectSetText("jpyp",DoubleToStr(value,1),8,"Arial Narrow",Silver);
  }
  
void paintLine()
  {
   ObjectSet("line1",OBJPROP_COLOR,DimGray);
   ObjectSet("line2",OBJPROP_COLOR,DimGray);
   ObjectSetText("time",Time1,9,"Arial",Yellow);
  }
  
void paintAlert()
  {
   ObjectSetText("Alert2",Alert2,10,"Arial",Red);
  }  
  
//============================================================================================================================
//============================================================================================================================


//END


